@@ -2,8 +2,9 @@ from .stream import BnpStream, NpDataclassStream
 from .decorators import streamable
 from .reductions import mean, bincount, histogram, quantile
 from .grouped import grouped_dict, grouped_stream
+from .groupby_func import groupby
 from .multistream import MultiStream
 
 __all__ = ["BnpStream", "streamable", "decorators",
            "mean", "bincount", "histogram", "quantile", "NpDataclassStream",
-           "BnpStream", "MultiStream"]
+           "BnpStream", "MultiStream", "groupby"]
