@@ -152,6 +152,10 @@ def test_read_gtf():
 
 @pytest.mark.parametrize("file_name", glob.glob("example_data/*"))
 def test_read_example_data(file_name):
+    if "broken" in file_name:
+        # broken data should not pass tests
+        return
+
     if file_name.endswith(".sam"):
         return
 
@@ -161,6 +165,9 @@ def test_read_example_data(file_name):
     if file_name.endswith(".jaspar"):
         return
 
+    if file_name.endswith(".tsv"):
+        return
+
     file = bnp.open(file_name)
     for chunk in file.read_chunks():
         continue
